//globally
//declare the min and max variables that you need in parseInfo
float minX, maxX;
float minY, maxY;
int totalCount; // total number of places
int minPopulation, maxPopulation;
int minSurface, maxSurface;
int minAltitude, maxAltitude;
//declare the variables corresponding to the column ids for x and y
int x = 1;
int y = 2;

City cities[];

int legendsHeight=100;
int minPopulationToDisplay=10000;
boolean redrawInProgress=false;

void setup() {
  size(900,900);
  colorMode(HSB, 360, 100, 100);
  readData();
  noLoop();
}

void draw(){
  background(255);
  
  // Draw title
  fill(0);
  textSize(20);
  textAlign(CENTER,CENTER);
  text("Afficher les populations supérieures à " + minPopulationToDisplay, width/2, 20);
  textSize(12);
  
  for (int i = 0 ; i < totalCount; ++i) {
    // draw a point at the coordinates of the city
    cities[i].draw();
  }

  updatePixels();
  
  // Draw the distribution of the altitude values
  float[] altitudeValues = new float[cities.length];
  for (int i = 0; i < totalCount; i++) {
    altitudeValues[i] = cities[i].altitude;
  }

  if (legendsHeight > 0) {
    drawLegend();
  }
  redrawInProgress=false;
}

void readData() {
  redrawInProgress=true;
  String[] lines = loadStrings("./villes.tsv");
  
  // check that the file has been loaded
  if (lines == null) {
    println("Error loading file");
    exit();
  } else {
    println("File loaded");
  }
  
  parseInfo(lines[0]); // read the header line
  
  ArrayList<City> filteredCities = new ArrayList<City>();

  for (int i = 2 ; i < totalCount ; ++i) {
    String[] columns = split(lines[i], TAB);
      float pointX = float (columns[x]);
      float pointY = float (columns[y]);
      // filter the cities
      if (float (columns[5]) > minPopulationToDisplay) {
        filteredCities.add(new City(columns[0], columns[4], mapX(pointX), mapY(pointY), float (columns[5]), float (columns[6]), float (columns[7])));
      }
  }
  
  totalCount = filteredCities.size();
  cities = filteredCities.toArray(new City[0]);
  println("City list created: " + totalCount + " cities");


}

void parseInfo(String line) {
  String infoString = line.substring(2); // remove the #
  String[] infoPieces = split(infoString, ',');
  totalCount = int(infoPieces[0]);
  minX = float(infoPieces[1]);
  maxX = float(infoPieces[2]);
  minY = float(infoPieces[3]);
  maxY = float(infoPieces[4]);
  minPopulation = int(infoPieces[5]);
  maxPopulation = int(infoPieces[6]);
  minSurface = int(infoPieces[7]);
  maxSurface = int(infoPieces[8]);
  minAltitude = int(infoPieces[9]);
  maxAltitude = int(infoPieces[10]);
}

float mapX(float x) {
 return map(x, minX, maxX, 50, 800);
}

float mapY(float y) {
  return map(y, maxY, minY, 50, 800);
}

void drawLegend() {
  float distributionX = 50;
  float distributionY = height - legendsHeight;
  float distributionWidth = width - 100;
  float distributionHeight = 50;
  // get list of population values from the cities
  float[] populationValues = new float[cities.length];
  for (int i = 0; i < cities.length - 2; i++) {
    populationValues[i] = cities[i].population;
  }

  int numberOfDiv = 30;
  // draw the population distribution
  //drawAltitudeLegend(minAltitude, maxAltitude, distributionX, distributionY, distributionWidth, distributionHeight);
  drawDistributionPopulationLegend(minPopulation, maxPopulation, distributionX, distributionY, distributionWidth, distributionHeight, numberOfDiv, populationValues);
}

void keyPressed() {
  if(redrawInProgress) {
    return;
  }
  int gap=2;
  if (key == '+') {
    if (minPopulationToDisplay == 0) {
      minPopulationToDisplay = 1;
    }
    minPopulationToDisplay = minPopulationToDisplay * gap;
    readData();
  } else if (key == '-') {
    minPopulationToDisplay = minPopulationToDisplay / gap;
    readData();
  }
  redraw();
}
